from django.contrib import admin

from .models import SavedSearch, AdvancedSavedSearch


class SavedSearchAdmin(admin.ModelAdmin):
    list_display = ('name', 'user', 'url', 'querystring', 'default', 'date_created')
    list_filter = ('url', 'user', 'default')

class AdvancedSavedSearchAdmin(admin.ModelAdmin):
    list_display = ('name', 'user', 'url', 'search_object', 'default', 'date_created')
    list_filter = ('url', 'user', 'default')


admin.site.register(SavedSearch, SavedSearchAdmin)
admin.site.register(AdvancedSavedSearch, AdvancedSavedSearchAdmin)



from django.forms import ModelForm

from .models import AdvancedSavedSearch, SavedSearch


class BaseSavedSearchForm(ModelForm):
    def __init__(self, *args, **kwargs):
        # saved_searches is a list of saved search objects that are in the same group that this form will be related to
        # By "same group" we mean are related to the same seeker instance, possibly the same user (depending on if that is enforced, etc.)
        self.saved_searches = kwargs.pop('saved_searches', [])
        self.enforce_unique_name = kwargs.pop('enforce_unique_name', True)
        super(BaseSavedSearchForm, self).__init__(*args, **kwargs)
        
    def save(self, commit=True):
        saved_search = super(BaseSavedSearchForm, self).save(commit=False)
        # We can only have one default in the group, so set the others to false
        # This queryset does not include the saved_search about to be saved
        if saved_search.default:
            self.saved_searches.update(default=False)
        
        # We enforce the naming restrictions here (depending on the 'unique_name_enforcement' setting)
        if self.enforce_unique_name:
            same_name_searches = self.saved_searches.filter(name=saved_search.name).delete()
            
        if commit:
            saved_search.save()
        return saved_search


class SavedSearchForm(BaseSavedSearchForm):
    class Meta:
        model = SavedSearch
        fields = ['name', 'default']

        
class AdvancedSavedSearchForm(BaseSavedSearchForm):
    class Meta:
        model = AdvancedSavedSearch
        fields = ['name', 'url', 'default', 'search_object']



# -*- coding: utf-8 -*-
# Generated by Django 1.10.8 on 2018-02-14 20:01
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('seeker', '0002_auto_20150507_0134'),
    ]

    operations = [
        migrations.AddField(
            model_name='savedsearch',
            name='data',
            field=models.TextField(blank=True),
        ),
        migrations.AddField(
            model_name='savedsearch',
            name='saved',
            field=models.BooleanField(default=False, help_text='This saved search was marked "saved" by the user.'),
        ),
        migrations.AlterField(
            model_name='savedsearch',
            name='name',
            field=models.CharField(blank=True, max_length=100),
        ),
    ]



from django.conf import settings
from django.db import connections
from django.db.models.sql.compiler import SQLCompiler
from django.db.models.sql.constants import MULTI
from django.db.models.sql.query import Query


def cursor_iter(cursor, fetch_size=1000):
    try:
        while True:
            cursor.execute('FETCH %s FROM seeker_cursor' % fetch_size)
            rows = cursor.fetchall()
            if not rows:
                raise StopIteration()
            yield rows
    finally:
        cursor.execute('ROLLBACK')
        cursor.close()


class CursorCompiler(SQLCompiler):

    def execute_sql(self, result_type=MULTI):
        if result_type != MULTI:
            raise ValueError('CursorCompiler can only be used for MULTI queries.')
        sql, params = self.as_sql()
        cursor = self.connection.cursor()
        cursor.execute('BEGIN')
        cursor.execute('DECLARE seeker_cursor CURSOR FOR ' + sql, params)
        return cursor_iter(cursor, fetch_size=getattr(settings, 'SEEKER_BATCH_SIZE', 1000))


class CursorQuery(Query):

    def get_compiler(self, using=None, connection=None):
        if using:
            connection = connections[using]
        return CursorCompiler(self, connection, using)
